import pygame
import game, gfx
import math
import vectormath
import objship

IDLE, ATTACK, FLEE = range(3)

class AI(objship.Ship):    
    def __init__(self, enemies=[], period=1):
        objship.Ship.__init__(self)
        self.enemies = enemies
        self.attackEnemy = None
        self.mode = IDLE
        self.period = period
        self.targetpos = [0, 0]
        self.objects = []
        self.threshold1 = 200
        self.threshold2 = 80
        self.switchToFleeModeThreshold = 25
        self.shootingCone = 60 #angle of cone in degrees within the enemy must lie to trigger shooting at enemy
        self.delta = 85
        self.time = 0
        self.isHit = False
        
    def addEnemy(self, enemy):
        self.enemies.append(enemy)
        
    def deleteEnemy(self, enemy):
        self.enemies.remove(enemy)
        
    def setPeriod(self, period):
        self.period = period
        
    def setTargetPosition(self, tpos):
        #self.targetpos = [tpos[0] - (self.shipDiameter / 2), tpos[1] - (self.shipDiameter / 2)]
        self.targetpos = tpos
        self.unmoved = 0
        
    def start(self, pos):
        self.setTargetPosition(pos)
        objship.Ship.start(self, pos)
        
    def tick(self, speedadjust = 1.0):
        currentTime = pygame.time.get_ticks()
        if (currentTime - self.time > 500):
            self.determineModeOfOperation()
            self.handleAttackMode()
            self.setTargetPosition(self.calcNewCoordinates())
            self.time = currentTime
            
            #handle shooting at enemy:
            if self.mode == ATTACK:
                angle = math.degrees(math.atan2(self.attackEnemy.pos[1] - self.pos[1], self.attackEnemy.pos[0] - self.pos[0]))
                angleToTarget = (angle + 180)
                angleShip = (self.angle - 180 ) % 360
        
                if(math.fabs(angleToTarget - angleShip) < (self.shootingCone/2)):
                    self.cmd_shoot()
             
        steering = False
        #if(self.mode == self.idle)
        #self.angle = math.degrees(math.atan2(self.targetpos[1] - self.pos[1], self.targetpos[0] - self.pos[0]))
        angle = math.degrees(math.atan2(self.targetpos[1] - self.pos[1], self.targetpos[0] - self.pos[0]))
        angleToTarget = (angle + 180)
        angleShip = (self.angle - 180 ) % 360
        
        if(math.fabs(angleToTarget - angleShip) > 180):
            angleToTarget = 360 - angleToTarget
        
        #print "angle to target: ", angleToTarget
        #print "angle ship: ", angleShip
        #print "dif: ", math.fabs(angleShip - angleToTarget)
        
        if(math.fabs(angleShip - angleToTarget) > objship.steeringsensitivity):
            steering = True
            if(angleShip > angleToTarget):
                self.angle = self.angle - objship.steeringsensitivity
            else:
                self.angle = self.angle + objship.steeringsensitivity

            
        distance = math.hypot(self.targetpos[0] - self.pos[0], self.targetpos[1] - self.pos[1])
        if((not steering) and (distance > (self.shipDiameter * 3))):
            self.acceleration = 1
        else:
            if(self.speed > 1):
                self.acceleration = -1
            else:
                self.acceleration = 0
                if(distance < 3):
                    self.speed = 0
                else:
                    self.speed = 1
            
        objship.Ship.tick(self, speedadjust)
    
    def setGameObjects(self, obj):
        self.objects = obj
        
    def calcNewCoordinates(self):
        # calculate distance between ship and all other game objects
        #dn = [vectormath.dist(self.pos, o.pos) for o in self.objects]
        
        # calculate distance between ship and trajectory of object if the distance is less than a threshold
        objects = [o for o in self.objects if vectormath.dist(self.pos, o.pos) < self.threshold1]
        trajectory_n = [vectormath.trajectoryPA(o.pos, o.angle) for o in objects]
        #d_trajectory_n = [vectormath.distanceP2LParameterForm(t, o.pos) for (t, o) in zip(trajectory_n, objects)]
        d_trajectory_n = [vectormath.distanceP2LParameterForm(t, self.pos) for t in trajectory_n]
        if (len(d_trajectory_n) == 0):
            # we cannot calculate a new desired target position, so we return the current one
            print 'Could not determine new target position. d_trajectory_n is empty'
            #return self.targetpos
            #if we have decided to attack an enemy and there is no collision with an object, we want to
            #set the enemies position as our new target position
            if self.attackEnemy != None:
                print 'attacking enemy at pos ', self.attackEnemy.pos
                return self.attackEnemy.pos
            else:
                return self.targetpos
        
        d_min = d_trajectory_n[0]
        trajectory_min = trajectory_n[0]
        
        for i, d_trajectory in enumerate(d_trajectory_n):
            if ((d_trajectory <= self.threshold2) and (d_min > d_trajectory)):
                d_min = d_trajectory
                trajectory_min = trajectory_n[i]
                
        if (d_min > self.delta):
            print 'No new target position. Distance to trajectory is greater than ', self.delta
            #return self.targetpos
            if self.attackEnemy != None:
                print 'attacking enemy at pos ', self.attackEnemy.pos
                return self.attackEnemy.pos
            else:
                return self.targetpos
        
        perpendicular = vectormath.perpendicular(self.pos, trajectory_min)
        p = vectormath.footOfPerpendicular(self.pos, trajectory_min)
                
        done = False
        dir = 0
        while not done:
            (q1, q2) = vectormath.pointWithDistanceDelta(perpendicular, p, self.delta)
            if dir == 1:
                q = q1
            elif dir == 2:
                q = q2
            else:
                if (vectormath.dist(q1, self.pos) < vectormath.dist(q2, self.pos)):
                    dir = 1
                    q = q1
                else:
                    dir = 2
                    q = q2
                
            done = True
            
            for t in trajectory_n:
                if ((not vectormath.isParallel(t, perpendicular)) and (vectormath.distanceP2LParameterForm(t, q) < self.threshold2)):
                    done = False
                    break
                
            p = q
        print 'Determined new target position: ', p
        return p
    
    def determineModeOfOperation(self):
        if self.mode == IDLE:
            if self.isHit == True:
                self.mode = ATTACK
        elif self.mode == ATTACK:
            if self.lifeEnergy <= self.switchToFleeModeThreshold:
                self.mode = FLEE
        elif self.mode == FLEE:
            #if there are enemies in a defined range...
            for e in self.enemies: 
                if vectormath.dist(self.pos, e.pos) < self.threshold1:
                    self.mode = IDLE      
        return
    
    def handleAttackMode(self):
        if self.mode != ATTACK:
            return
        
        #decide which enemy to attack: attack nearest enemy
        self.attackEnemy = None
        dist = math.hypot(game.arena.width, game.arena.height)
        for e in self.enemies:
            disttemp = vectormath.dist(self.pos, e.pos)
            if disttemp < dist:
                dist = disttemp
                self.attackEnemy = e
    
class cursor:
    def __init__(self):
        self.pos = [game.arena.width / 2, game.arena.height / 2]
        self.img = createCursorImage()
        self.rect = self.img.get_rect()
        
    def createCursorImage(self):
        surface = pygame.Surface([10, 10])
        pygame.draw.circle(surface, (0xff, 0xff, 0xff), (5, 5), 5, 1)
        return surface
    
    def erase(self, background):
        r = background(self.rect)
        #if self.dead:
        gfx.dirty(r)

        
    def draw(self, gfx):
        r = gfx.surface.blit(self.img, self.rect)
        gfx.dirty(r)    