"""Game start and user select handler, part of SOLARFighter."""
#created by Adrian Posor
import pygame
import game
import gfx, txt, snd
import input
import players
import gameplay

images = []
delimage = None
namefont = None
textfont = None
smallfont = None
lastplayer = None
gametype = 0


def load_game_resources():
    global images, namefont, textfont, smallfont, delimage

    img = pygame.transform.rotate(gfx.load('ship-up.png'), -90)
    images.append((img, img.get_rect()))

    font = txt.Font(None, 50)
    t = font.text((220, 210, 180), 'Choose a Type of Game:', (gfx.rect.centerx, 30))
    images.append(t)

    namefont = txt.Font(None, 46)
    textfont = txt.Font(None, 26)
    smallfont = txt.Font(None, 16)

    snd.preload('select_choose', 'select_move', 'delete')

    delimage = gfx.load('btn-delete.gif')




class GameTypeMenu:
    def __init__(self, prevhandler):
        self.prevhandler = prevhandler
        self.images = images
        self.done = 0
        self.aborted = 0
        self.menulabels = ['SolarFighter', 'SolarFighter meets SolarWolf', 'SolarWolf 360']
        self.menu = []
        self.current = [0, 0]
        self.buildlist()
        self.current = [0, 0]
        self.shipmovex = 40
        self.shipmovey = 16

        step = 0
        if lastplayer:
            for g in self.menu:
                if players.find_player(g[0]) == lastplayer:
                    self.current[0] = step
                    self.buildlist()
                step += 1

        self.moveto(self.menu[self.current[0]][1][1])



    def starting(self):
        #snd.playmusic('gamestart.wav')
        pass


    def moveto(self, pos):
        self.shippos = pos[0] - 10, pos[1] + 10


    def input(self, i):
        if self.done:
            return
        if i.release:
            return
        if i.translated == input.ABORT:
            self.aborted = 1
            self.done = 1
            self.current[0] = -1
            self.clearlist()
            self.moveto((2, 2))
            snd.play('select_choose')
        if i.translated == input.PRESS:
            return self.pressed()

        if i.translated in ( input.DOWN, input.UP, input.LEFT, input.RIGHT):
            if i.translated == input.DOWN:
                self.current[0] = (self.current[0] + 1) % len(self.menu)
            elif i.translated == input.UP:
                self.current[0] = (self.current[0] - 1) % len(self.menu)
            elif i.translated == input.LEFT or i.translated == input.RIGHT:
                self.current[1] = not self.current[1]

            if self.current[1] and len(self.menu[self.current[0]])<=2:
                self.current[1] = 0
            snd.play('select_move')
            x = self.menu[self.current[0]][1+self.current[1]][1].left
            y = self.menu[self.current[0]][1][1].top + self.current[1]* 10
            self.moveto((x, y))
            self.buildlist()

    def event(self, e):
        pass

    def run(self):
        r = self.background(self.images[0][1])
        gfx.dirty(r)

        self.moveship()
        gfx.updatestars(self.background, gfx)

        if not self.done:
            self.drawlist()
            for img in self.images:
                r = gfx.surface.blit(img[0], img[1])
                gfx.dirty(r)
        else:
            if gfx.rect.colliderect(self.images[0][1]) and not self.aborted:
                if self.current[0] >= 0:
                    g = self.menu[self.current[0]]
                    r = gfx.surface.blit(g[1][0], g[1][1])
                    gfx.dirty(r)
                for img in self.images:
                    r = gfx.surface.blit(img[0], img[1])
                    gfx.dirty(r)
            else:
                self.clearlist()
                if self.aborted:
                    game.handler = self.prevhandler
                else:
                    game.handler = gameplay.GamePlay(self.prevhandler)

            for img in self.images[1:]:
                r = self.background(img[1])
                gfx.dirty(r)



    def background(self, area):
        return gfx.surface.fill((0, 0, 0), area)


    def buildlist(self):
        self.clearlist()
        clr = 160, 200, 250
        clr2 = 200, 200, 200
        clr3 = 100, 160, 255
        offsetx, offsety = 120, 110
        sizey = 85
        self.clearlist()
        self.menu = []
        cur = 0
        for menuitem in self.menulabels:
            t = namefont.text(clr, menuitem, (400, offsety), 'center')
            self.menu.append(('', t))
            offsety += sizey
            cur += 1
 

    def clearlist(self):
        for g in self.menu:
            for i,r in g[1:]:
                gfx.dirty(self.background(r))

    def drawlist(self):
        for g in self.menu:
            for b in g[1:]:
                gfx.dirty(gfx.surface.blit(*b))

    def moveship(self):
        pos = list(self.images[0][1].topright)
        if pos[0] + self.shipmovex < self.shippos[0]:
            pos[0] += self.shipmovex
        elif pos[0] < self.shippos[0]:
            pos[0] = self.shippos[0]

        if pos[0] - self.shipmovex > self.shippos[0]:
            pos[0] -= self.shipmovex
        elif pos[0] > self.shippos[0]:
            pos[0] = self.shippos[0]

        if pos[1] + self.shipmovey < self.shippos[1]:
            pos[1] += self.shipmovey
        elif pos[1] < self.shippos[1]:
            pos[1] = self.shippos[1]

        if pos[1] - self.shipmovey > self.shippos[1]:
            pos[1] -= self.shipmovey
        elif pos[1] > self.shippos[1]:
            pos[1] = self.shippos[1]

        self.images[0][1].topright = pos


    def pressed(self):
        global lastplayer, gametype
        p = players.Player('')
        lastplayer = p
        game.player = p
        self.moveto((gfx.surface.get_size()[0] + 60, 500))
        self.clearlist()
        self.done = 1
        gametype = self.current[0]
        print 'gametype: ', gametype
#        g = self.menu[self.current[0]]
#        p = players.find_player(g[0])
#        if self.current[1] == 0:
#            snd.play('select_choose')
#            if not p:
#                p = players.Player('')
#            global lastplayer
#            lastplayer = p
#            game.player = p
#            self.clearlist()
#            #self.moveto((860, 500))
#            self.moveto((gfx.surface.get_size()[0] + 60, 500))
#            self.done = 1
#        else:
#            snd.play('delete')
#            players.players.remove(p)
#            self.buildlist()
#            self.current = [0, 0]
#            self.moveto((120, 110))


