import math

class lineHNF:
    def __init__(self, r, n, c):
        self.r = r
        self.n = n
        self.d = c
        
class lineParameterForm:
    def __init__(self, a, b):
        self.a = a
        self.b = b

        
# This function calculates the distance between two points
# Algorithm 3
def dist(p, q):
    deltaX = math.fabs(p[0] - q[0])
    deltaY = math.fabs(p[1] - q[1])
    d = math.sqrt(deltaX**2 + deltaY**2)
    return d   

# This function calculates the length of a vector
def abs(p):
    return math.sqrt(p[0]**2 + p[1]**2)

# This function calculates the distance between a point and a line in HNF
# Algorithm 4
def distanceP2LHNF(point, line):
    return dist(dotProduct(line.n, point) - line.c)

# This function calculates the distance between a point and a line
# in parameter form. Algorithm 5
def distanceP2LParameterForm(lpf, p):
    a = lpf.a
    b = lpf.b
    u = ((p[0] - a[0]) * b[0] + (p[1] - a[1]) * b[1]) / (b[0] * b[0] + b[1] * b[1])
    closestPoint = (a[0] + u * b[0], a[1] + u * b[1])
    return math.hypot(closestPoint[0] - p[0], closestPoint[1] - p[1])
  
# This function calculates the trajectory of an object (calculates line given a point
# and an angle). The result is a line in parameter form. The angle is in radians.
# Algorithm 6
def trajectoryPA(point, angle):
    return lineParameterForm(point, (math.cos(angle), math.sin(angle)))

# This function calculates the trajectory (line) given two points. The result
# is a line in parameter form. Algorithm 7
def trajectoryPP(point1, point2):
    return lineParameterForm(point1, vectorSubstraction(point2, point1))

# This function calculates the perpendicular of the given line which goes
# through the given point. Algorithm 8
def perpendicular(point, line):
    p = point
    a = line.a
    b = line.b
    t0 = ((p[0] - a[0]) * b[0] + (p[1] - a[1]) * b[1]) / (b[0] * b[0] + b[1] * b[1])
    x0 = (a[0] + t0 * b[0], a[1] + t0 * b[1])
    return lineParameterForm(point, vectorSubstraction(x0, p))

# This function calculates the foot of the perpendicular of the given line which goes
# through the given point. Algorithm 9
def footOfPerpendicular(point, line):
    p = point
    a = line.a
    b = line.b
    t0 = ((p[0] - a[0]) * b[0] + (p[1] - a[1]) * b[1]) / (b[0] * b[0] + b[1] * b[1])
    x0 = (a[0] + t0 * b[0], a[1] + t0 * b[1])
    return x0

def dotProduct(p, q):
    return sum([x*y for (x, y) in zip(p, q)])

def crossProduct(p, q):
    if (len(p) == len(q) == 3):
        return (p[1] * q[2] - p[2] * q[1], p[2] * q[0] - p[0] * q[2], p[0] * q[1] - p[1] * q[0])
    else:
        raise TypeError('Vector of incorrect dimension')
    
# This function checks if two lines given in parameter form are orthogonal to each other.
# It returns 'true' if the lines are orthogonal, 'false' otherwise
def isOrthogonal(line1, line2):
    return dotProduct(line1.b, line2.b) < 0.0001

# This function checks if two lines given in parameter form are parallel.
# It returns 'true' if the lines are parallel, 'false' otherwise.
def isParallel(line1, line2):
    #return angleBetweenLines(line1, line2) == 0
    return perpDotProduct(line1.b, line2.b) < 0.0001
    
# This function calculates the angle in radians between the two given lines in parameter form.
def angleBetweenLines(line1, line2):
    return math.acos(math.fabs(dotProduct(line1.b, line2.b) / (abs(line1.b) * abs(line2.b))))

# This function calculates the perpendicular dot product. This is a modified version of the dot product
# for vectors in 2D in which one vector is replaced by the perpendicular vector rotated 90 degrees
# to the left as defined by Hill (1994).
def perpDotProduct(p, q):
    return dotProduct((-p[1], p[0]), q)
    
# This function calculates the two points P2 on the line that have the distance delta to point P1
def pointWithDistanceDelta(line, point, delta):
    a1 = line.a[0]
    a2 = line.a[1]
    b1 = line.b[0]
    b2 = line.b[1]
    x1 = point[0]
    y1 = point[1]
    
    t1 = (math.sqrt(-a1**2 * b2**2 + 2 * a1 * (a2 * b1 - b1 * y1 + b2 * x1) * b2 - a2**2 * b1**2 + 2 * a2 * b1 * (b1 * y1 - b2 * x1) + b1**2 * (delta**2 - y1**2) + 2 * b1 * b2 * x1 * y1 +b2**2 * (delta**2 - x1**2)) - a1 * b1 - a2 * b2 + b1 * x1 + b2 * y1) / (b1**2 + b2**2)
    t2 = -(math.sqrt(-a1**2 * b2**2 + 2 * a1 * (a2 * b1 - b1 * y1 + b2 * x1) * b2 - a2**2 * b1**2 + 2 * a2 * b1 * (b1 * y1 - b2 * x1) + b1**2 * (delta**2 - y1**2) + 2 * b1 * b2 * x1 * y1 +b2**2 * (delta**2 - x1**2)) + a1 * b1 + a2 * b2 - b1 * x1 - b2 * y1) / (b1**2 + b2**2)
    return ((a1 + t1 * b1, a2 + t1 * b2), (a1 + t2 * b1, a2 + t2 * b2)) 
        
def vectorSum(p, q):
    return (p[0] + q[0], p[1] + q[1])
            
def vectorSubstraction(p, q):
    return (p[0] - q[0], p[1] - q[1])